==> nux_0.9.48.orig.tar.gz: nux-0.9.48/Nux/ValuatorDouble.cpp <==
/*
 * Copyright 2010 Inalogic® Inc.
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License, as
 * published by the  Free Software Foundation; either version 2.1 or 3.0
 * of the License.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranties of
 * MERCHANTABILITY, SATISFACTORY QUALITY or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the applicable version of the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of both the GNU Lesser General Public
 * License along with this program. If not, see <http://www.gnu.org/licenses/>
 *
 * Authored by: Jay Taoko <jaytaoko@inalogic.com>
 *
 */


#include "Nux.h"
#include "TimerProc.h"
#include "WindowCompositor.h"
#include "ValuatorDouble.h"

namespace nux
{

  const Color SPINBOX_BUTTON_COLOR = Color (0xFF4D4D4D);
  const Color SPINBOX_BUTTON_MOUSEOVER_COLOR = Color (0xFF222222);

  ValuatorDouble::ValuatorDouble (double Value, double Step, double MinValue, double MaxValue)
    :   m_DoubleValidator (MinValue, MaxValue)
    ,   m_Step (Step)
  {
    m_EditLine->SetValidator (&m_DoubleValidator);
    m_EditLine->SetSuffix (TEXT ("") );
    m_EditLine->SetPrefix (TEXT ("") );

    NString s = NString::Printf (TEXT ("%.3f"), m_DoubleValidator.GetMinimum ());
    m_EditLine->SetText (s);

    m_EditLine->SetMinimumSize (1.5 * DEFAULT_WIDGET_WIDTH, DEFAULT_WIDGET_HEIGHT);
    m_EditLine->SetGeometry (Geometry (0, 0, DEFAULT_WIDGET_WIDTH, DEFAULT_WIDGET_HEIGHT) );

    m_MouseControlledButton->SetMinimumSize (16 + 9, 10);
    m_MouseControlledButton->SetGeometry (Geometry (0, 0, 15, 10) );

    // Set the minimum size of this widget.
    // This is use by TextLineEditPropertyItem::GetItemBestHeight
    SetMinimumSize (DEFAULT_WIDGET_WIDTH, PRACTICAL_WIDGET_HEIGHT);

    m_hlayout = new HLayout (NUX_TRACKER_LOCATION);

    m_hlayout->AddView (m_MouseControlledButton, 0);
    m_hlayout->AddView (m_EditLine, 1);
    m_hlayout->SetHorizontalInternalMargin (4);
    SetCompositionLayout (m_hlayout);

    SetValue (Value);
  }

  ValuatorDouble::~ValuatorDouble()
  {
  }

  long ValuatorDouble::ProcessEvent (IEvent &ievent, long TraverseInfo, long ProcessEventInfo)
  {
    long ret = TraverseInfo;
    ret = m_MouseControlledButton->OnEvent (ievent, ret, ProcessEventInfo);
    ret = m_EditLine->ProcessEvent (ievent, ret, ProcessEventInfo);
    ret = PostProcessEvent2 (ievent, ret, ProcessEventInfo);
    return ret;
  }

  void ValuatorDouble::Draw (GraphicsEngine &GfxContext, bool force_draw)
  {
    Geometry base = GetGeometry();
    GetPainter().PaintBackground (GfxContext, base);

    if (m_EditLine->IsMouseInside() ||
        m_MouseControlledButton->HasMouseFocus() || m_MouseControlledButton->IsMouseInside() )
    {

      GetPainter().PaintShapeCorner (GfxContext, base, SPINBOX_BUTTON_MOUSEOVER_COLOR, eSHAPE_CORNER_ROUND4,
                                 eCornerTopLeft | eCornerBottomLeft, false);
    }
    else
    {
      GetPainter().PaintShapeCorner (GfxContext, base, SPINBOX_BUTTON_COLOR, eSHAPE_CORNER_ROUND4,
                                 eCornerTopLeft | eCornerBottomLeft, false);
    }

    GeometryPositioning gp (eHACenter, eVACenter);
    Geometry GeoPo = ComputeGeometryPositioning (m_MouseControlledButton->GetGeometry(), GetTheme().GetImageGeometry (eVALUATORMOVE), gp);

    if (m_Mouse == 1)
    {
      if (m_MouseControlledButton->IsMouseInside() )
        GetPainter().PaintShape (GfxContext, GeoPo, Color (0xFFFFFFFF), eVALUATORHORIZONTALMOVE);
      else
        GetPainter().PaintShape (GfxContext, GeoPo, Color (0xFFFFFFFF), eVALUATORHORIZONTALMOVE);
    }
    else if (m_Mouse == 3)
    {
      if (m_MouseControlledButton->IsMouseInside() )
        GetPainter().PaintShape (GfxContext, GeoPo, Color (0xFFFFFFFF), eVALUATORVERTICALMOVE);
      else
        GetPainter().PaintShape (GfxContext, GeoPo, Color (0xFFFFFFFF), eVALUATORVERTICALMOVE);
    }
    else
    {
      if (m_MouseControlledButton->IsMouseInside() )
        GetPainter().PaintShape (GfxContext, GeoPo, Color (0xFFFFFFFF), eVALUATORMOVE);
      else
        GetPainter().PaintShape (GfxContext, GeoPo, Color (0xFFFFFFFF), eVALUATORMOVE);
    }



    Geometry geo = m_EditLine->GetGeometry();
    geo.OffsetPosition (-4, 0);
    GetPainter().PaintShapeCorner (GfxContext, geo, m_EditLine->GetTextBackgroundColor(), eSHAPE_CORNER_ROUND4,
                               eCornerTopLeft | eCornerBottomLeft, false);

    m_EditLine->NeedRedraw();
  }

  void ValuatorDouble::DrawContent (GraphicsEngine &GfxContext, bool force_draw)
  {
    m_EditLine->ProcessDraw (GfxContext, force_draw);
  }

  void ValuatorDouble::PostDraw (GraphicsEngine &GfxContext, bool force_draw)
  {

  }

  void ValuatorDouble::SetValue (double value)
  {
    m_Value = m_DoubleValidator.GetClampedValue (value);
    m_EditLine->SetText (NString::Printf ("%.3f", m_Value) );
    sigValueChanged.emit (this);
    sigValue.emit (m_Value);
    NeedRedraw();
  }

  double ValuatorDouble::GetValue() const
  {
    return m_Value;
  }

  void ValuatorDouble::SetStep (double i)
  {
    m_Step = i;

    if (m_Step <= 0)
      m_Step = 1;

    NeedRedraw();
  }

  double ValuatorDouble::GetStep() const
  {
    return m_Step;
  }

  double ValuatorDouble::GetMinValue() const
  {
    return m_DoubleValidator.GetMinimum();
  }

  double ValuatorDouble::GetMaxValue() const
  {
    return m_DoubleValidator.GetMaximum();
  }

  void ValuatorDouble::SetRange (double MinValue, double Maxvalue)
  {
    m_DoubleValidator.SetMinimum (MinValue);
    m_DoubleValidator.SetMaximum (Maxvalue);
    m_Value = m_DoubleValidator.GetClampedValue (m_Value);
    sigValueChanged.emit (this);
    sigValue.emit (m_Value);
    NeedRedraw();
  }

  void ValuatorDouble::ImplementIncrementBtn()
  {
    SetValue (m_Value + m_Step);

    if (m_Value < m_DoubleValidator.GetMaximum() )
    {
      NeedRedraw();
    }

    sigValueChanged.emit (this);
    sigIncrement.emit (this);
    sigValue.emit (m_Value);
  }

  void ValuatorDouble::ImplementDecrementBtn()
  {
    SetValue (m_Value - m_Step);

    if (m_Value > m_DoubleValidator.GetMinimum() )
    {
      NeedRedraw();
    }

    sigValueChanged.emit (this);
    sigDecrement.emit (this);
    sigValue.emit (m_Value);
  }

  void ValuatorDouble::ImplementValidateEntry()
  {
    DOUBLE ret = 0;
    ret = CharToDouble (m_EditLine->GetCleanText().GetTCharPtr() );
    {
      m_Value = m_DoubleValidator.GetClampedValue (ret);
      m_EditLine->SetText (NString::Printf ("%.3f", m_Value) );
      sigValueChanged.emit (this);
      sigValue.emit (m_Value);
//
//        if(m_Value < m_DoubleValidator.GetMinimum())
//        {
//            m_Value = m_DoubleValidator.GetMinimum();
//            m_EditLine->SetText(NString::Printf("%.3f", m_Value));
//        }
//        if(m_Value > m_DoubleValidator.GetMaximum())
//        {
//            m_Value = m_DoubleValidator.GetMaximum();
//            m_EditLine->SetText(NString::Printf("%.3f", m_Value));
//        }
    }
//     else
//     {
//         m_EditLine->SetText(NString::Printf("%.3f", m_Value));
//         sigValueChanged.emit(this);
//         sigValue.emit(m_Value);
//     }
  }



}

==> nux_1.16.0.orig.tar.gz: nux-1.16.0/Nux/ValuatorDouble.cpp <==
/*
 * Copyright 2010 Inalogic® Inc.
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License, as
 * published by the  Free Software Foundation; either version 2.1 or 3.0
 * of the License.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranties of
 * MERCHANTABILITY, SATISFACTORY QUALITY or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the applicable version of the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of both the GNU Lesser General Public
 * License along with this program. If not, see <http://www.gnu.org/licenses/>
 *
 * Authored by: Jay Taoko <jaytaoko@inalogic.com>
 *
 */


#include "Nux.h"
#include "TimerProc.h"
#include "WindowCompositor.h"
#include "ValuatorDouble.h"

namespace nux
{

  const Color SPINBOX_BUTTON_COLOR = Color (0xFF4D4D4D);
  const Color SPINBOX_BUTTON_MOUSEOVER_COLOR = Color (0xFF222222);

  ValuatorDouble::ValuatorDouble (double Value, double Step, double MinValue, double MaxValue)
    :   m_DoubleValidator (MinValue, MaxValue)
    ,   m_Step (Step)
  {
    m_EditLine->SetValidator (&m_DoubleValidator);
    m_EditLine->SetSuffix (TEXT ("") );
    m_EditLine->SetPrefix (TEXT ("") );

    NString s = NString::Printf (TEXT ("%.3f"), m_DoubleValidator.GetMinimum ());
    m_EditLine->SetText (s);

    m_EditLine->SetMinimumSize (1.5 * DEFAULT_WIDGET_WIDTH, DEFAULT_WIDGET_HEIGHT);
    m_EditLine->SetGeometry (Geometry (0, 0, DEFAULT_WIDGET_WIDTH, DEFAULT_WIDGET_HEIGHT) );

    m_MouseControlledButton->SetMinimumSize (16 + 9, 10);
    m_MouseControlledButton->SetGeometry (Geometry (0, 0, 15, 10) );

    // Set the minimum size of this widget.
    // This is use by TextLineEditPropertyItem::GetItemBestHeight
    SetMinimumSize (DEFAULT_WIDGET_WIDTH, PRACTICAL_WIDGET_HEIGHT);

    m_hlayout = new HLayout (NUX_TRACKER_LOCATION);

    m_hlayout->AddView (m_MouseControlledButton, 0);
    m_hlayout->AddView (m_EditLine, 1);
    m_hlayout->SetHorizontalInternalMargin (4);
    SetCompositionLayout (m_hlayout);

    SetValue (Value);
  }

  ValuatorDouble::~ValuatorDouble()
  {
  }

  long ValuatorDouble::ProcessEvent (IEvent &ievent, long TraverseInfo, long ProcessEventInfo)
  {
    long ret = TraverseInfo;
    ret = m_MouseControlledButton->OnEvent (ievent, ret, ProcessEventInfo);
    ret = m_EditLine->ProcessEvent (ievent, ret, ProcessEventInfo);
    ret = PostProcessEvent2 (ievent, ret, ProcessEventInfo);
    return ret;
  }

  void ValuatorDouble::Draw (GraphicsEngine &GfxContext, bool force_draw)
  {
    Geometry base = GetGeometry();
    GetPainter().PaintBackground (GfxContext, base);

    if (m_EditLine->IsMouseInside() ||
        m_MouseControlledButton->IsMouseOwner() || m_MouseControlledButton->IsMouseInside() )
    {

      GetPainter().PaintShapeCorner (GfxContext, base, SPINBOX_BUTTON_MOUSEOVER_COLOR, eSHAPE_CORNER_ROUND4,
                                 eCornerTopLeft | eCornerBottomLeft, false);
    }
    else
    {
      GetPainter().PaintShapeCorner (GfxContext, base, SPINBOX_BUTTON_COLOR, eSHAPE_CORNER_ROUND4,
                                 eCornerTopLeft | eCornerBottomLeft, false);
    }

    GeometryPositioning gp (eHACenter, eVACenter);
    Geometry GeoPo = ComputeGeometryPositioning (m_MouseControlledButton->GetGeometry(), GetTheme().GetImageGeometry (eVALUATORMOVE), gp);

    if (m_Mouse == 1)
    {
      if (m_MouseControlledButton->IsMouseInside() )
        GetPainter().PaintShape (GfxContext, GeoPo, Color (0xFFFFFFFF), eVALUATORHORIZONTALMOVE);
      else
        GetPainter().PaintShape (GfxContext, GeoPo, Color (0xFFFFFFFF), eVALUATORHORIZONTALMOVE);
    }
    else if (m_Mouse == 3)
    {
      if (m_MouseControlledButton->IsMouseInside() )
        GetPainter().PaintShape (GfxContext, GeoPo, Color (0xFFFFFFFF), eVALUATORVERTICALMOVE);
      else
        GetPainter().PaintShape (GfxContext, GeoPo, Color (0xFFFFFFFF), eVALUATORVERTICALMOVE);
    }
    else
    {
      if (m_MouseControlledButton->IsMouseInside() )
        GetPainter().PaintShape (GfxContext, GeoPo, Color (0xFFFFFFFF), eVALUATORMOVE);
      else
        GetPainter().PaintShape (GfxContext, GeoPo, Color (0xFFFFFFFF), eVALUATORMOVE);
    }



    Geometry geo = m_EditLine->GetGeometry();
    geo.OffsetPosition (-4, 0);
    GetPainter().PaintShapeCorner (GfxContext, geo, m_EditLine->GetTextBackgroundColor(), eSHAPE_CORNER_ROUND4,
                               eCornerTopLeft | eCornerBottomLeft, false);

    m_EditLine->QueueDraw();
  }

  void ValuatorDouble::DrawContent (GraphicsEngine &GfxContext, bool force_draw)
  {
    m_EditLine->ProcessDraw (GfxContext, force_draw);
  }

  void ValuatorDouble::PostDraw (GraphicsEngine &GfxContext, bool force_draw)
  {

  }

  void ValuatorDouble::SetValue (double value)
  {
    m_Value = m_DoubleValidator.GetClampedValue (value);
    m_EditLine->SetText (NString::Printf ("%.3f", m_Value) );
    sigValueChanged.emit (this);
    sigValue.emit (m_Value);
    QueueDraw();
  }

  double ValuatorDouble::GetValue() const
  {
    return m_Value;
  }

  void ValuatorDouble::SetStep (double i)
  {
    m_Step = i;

    if (m_Step <= 0)
      m_Step = 1;

    QueueDraw();
  }

  double ValuatorDouble::GetStep() const
  {
    return m_Step;
  }

  double ValuatorDouble::GetMinValue() const
  {
    return m_DoubleValidator.GetMinimum();
  }

  double ValuatorDouble::GetMaxValue() const
  {
    return m_DoubleValidator.GetMaximum();
  }

  void ValuatorDouble::SetRange (double MinValue, double Maxvalue)
  {
    m_DoubleValidator.SetMinimum (MinValue);
    m_DoubleValidator.SetMaximum (Maxvalue);
    m_Value = m_DoubleValidator.GetClampedValue (m_Value);
    sigValueChanged.emit (this);
    sigValue.emit (m_Value);
    QueueDraw();
  }

  void ValuatorDouble::ImplementIncrementBtn()
  {
    SetValue (m_Value + m_Step);

    if (m_Value < m_DoubleValidator.GetMaximum() )
    {
      QueueDraw();
    }

    sigValueChanged.emit (this);
    sigIncrement.emit (this);
    sigValue.emit (m_Value);
  }

  void ValuatorDouble::ImplementDecrementBtn()
  {
    SetValue (m_Value - m_Step);

    if (m_Value > m_DoubleValidator.GetMinimum() )
    {
      QueueDraw();
    }

    sigValueChanged.emit (this);
    sigDecrement.emit (this);
    sigValue.emit (m_Value);
  }

  void ValuatorDouble::ImplementValidateEntry()
  {
    DOUBLE ret = 0;
    ret = CharToDouble (m_EditLine->GetCleanText().GetTCharPtr() );
    {
      m_Value = m_DoubleValidator.GetClampedValue (ret);
      m_EditLine->SetText (NString::Printf ("%.3f", m_Value) );
      sigValueChanged.emit (this);
      sigValue.emit (m_Value);
//
//        if(m_Value < m_DoubleValidator.GetMinimum())
//        {
//            m_Value = m_DoubleValidator.GetMinimum();
//            m_EditLine->SetText(NString::Printf("%.3f", m_Value));
//        }
//        if(m_Value > m_DoubleValidator.GetMaximum())
//        {
//            m_Value = m_DoubleValidator.GetMaximum();
//            m_EditLine->SetText(NString::Printf("%.3f", m_Value));
//        }
    }
//     else
//     {
//         m_EditLine->SetText(NString::Printf("%.3f", m_Value));
//         sigValueChanged.emit(this);
//         sigValue.emit(m_Value);
//     }
  }



}
